@@ -3,11 +3,12 @@
 from logging import getLogger
 from numpy import (absolute, arange, argmax, argmin, asarray, concatenate, cos,
                    diff, exp, empty, floor, histogram, hstack, insert, invert,
-                   logical_and, mean, median, nan, ones, pi, ptp, real, sqrt, 
-                   square, std, vstack, where, zeros)
+                   logical_and, mean, median, nan, ones, percentile, pi, ptp, 
+                   real, sqrt, square, std, vstack, where, zeros)
 from scipy.ndimage.filters import gaussian_filter
 from scipy.signal import (argrelmax, butter, cheby2, filtfilt, 
-                          fftconvolve, hilbert, periodogram, remez, tukey)
+                          fftconvolve, hilbert, periodogram, remez, 
+                          sosfiltfilt, tukey)
 try:
     from PyQt5.QtCore import Qt
     from PyQt5.QtWidgets import QProgressDialog
@@ -33,7 +34,7 @@ class DetectSpindle:
     duration : tuple of float
         min and max duration of spindles
     """
-    def __init__(self, method='Moelle2002', frequency=None, duration=None,
+    def __init__(self, method='Moelle2011', frequency=None, duration=None,
                  merge=True):
 
         self.method = method
@@ -44,22 +45,7 @@ class DetectSpindle:
         self.frequency = frequency
         self.rolloff = 0
 
-        if method == 'Moelle2002':
-            if self.frequency is None:
-                self.frequency = (12, 15)
-            self.rolloff = 1.7
-            self.det_remez = {'freq': self.frequency,
-                              'rolloff': self.rolloff,
-                              'dur': 1.18
-                               }
-            self.duration = (0.5, 3)
-            self.det_wavelet = {'sd': None}
-            self.det_thresh_lo = 1.5
-            self.sel_thresh = None
-            self.moving_rms = {'dur': .2}
-            self.smooth = {'dur': .2}
-
-        elif method == 'Ferrarelli2007':
+        if method == 'Ferrarelli2007':
             if self.frequency is None:
                 self.frequency = (11, 15)
             self.rolloff = 0.9
@@ -88,6 +74,21 @@ class DetectSpindle:
             self.moving_rms = {'dur': None}
             self.smooth = {'dur': .04}  # is in fact sigma
 
+        elif method == 'Moelle2011':
+            if self.frequency is None:
+                self.frequency = (9, 15)
+            self.rolloff = 1.7
+            self.det_remez = {'freq': self.frequency,
+                              'rolloff': self.rolloff,
+                              'dur': 1.18
+                               }
+            self.duration = (0.5, 3)
+            self.det_wavelet = {'sd': None}
+            self.det_thresh_lo = 1.5
+            self.sel_thresh = None
+            self.moving_rms = {'dur': .2}
+            self.smooth = {'dur': .2}
+        
         elif method == 'Wamsley2012':
             if self.frequency is None:
                 self.frequency = (12, 15)
@@ -101,6 +102,35 @@ class DetectSpindle:
             self.moving_rms = {'dur': None}
             self.smooth = {'dur': .1}
 
+        elif method == 'Ray2015':
+            if self.frequency is None:
+                self.frequency = (11, 16)
+            self.cdemod = {'freq': mean(self.frequency)}
+            self.det_butter = {'freq': (0.3, 35),
+                               'order': 4}
+            self.det_low_butter = {'freq': 5,
+                                   'order': 4}
+            self.duration = (.49, None)
+            self.det_wavelet = {'sd': None}
+            self.det_thresh_lo = 2.33
+            self.sel_thresh = 0.1
+            self.smooth = {'dur': 2 / self.cdemod['freq']}
+            self.min_interval = 0.25 # they only start looking again after .25s
+            self.zwin = {'dur': 60}
+        
+        elif 'FASST' in method:
+            if self.frequency is None:
+                self.frequency = (11, 18)
+            self.det_butter = {'freq': self.frequency,
+                               'order': 4}
+            self.duration = (.4, 1.3)
+            self.det_wavelet = {'sd': None}
+            self.det_thresh_lo = 90
+            self.sel_thresh = None
+            self.moving_rms = {'dur': .1}
+            self.smooth = {'dur': .1}
+            self.min_interval = 1
+        
         elif method == 'UCSD':
             if self.frequency is None:
                 self.frequency = (10, 16)
@@ -201,7 +231,7 @@ class DetectSpindle:
                 sp_in_chan, values, density = detect_Nir2011(dat_orig,
                                                              data.s_freq,
                                                              time, self)
-
+            
             elif self.method == 'Wamsley2012':
                 sp_in_chan, values, density = detect_Wamsley2012(dat_orig,
                                                                  data.s_freq,
@@ -211,10 +241,27 @@ class DetectSpindle:
                 sp_in_chan, values, density = detect_UCSD(dat_orig,
                                                           data.s_freq, time,
                                                           self)
-            elif self.method == 'Moelle2002':
-                sp_in_chan, values, density = detect_Moelle2002(dat_orig,
+            elif self.method == 'Moelle2011':
+                sp_in_chan, values, density = detect_Moelle2011(dat_orig,
                                                                 data.s_freq,
                                                                 time, self)
+                
+            elif self.method == 'Ray2015':
+                sp_in_chan, values, density = detect_Ray2015(dat_orig,
+                                                            data.s_freq,
+                                                            time, self)
+                
+            elif self.method == 'FASST':
+                sp_in_chan, values, density = detect_FASST(dat_orig,
+                                                           data.s_freq,
+                                                           time, self,
+                                                           submethod='abs')
+                
+            elif self.method == 'FASST2':
+                sp_in_chan, values, density = detect_FASST(dat_orig,
+                                                           data.s_freq,
+                                                           time, self,
+                                                           submethod='rms')            
             elif self.method == 'Concordia':
                 sp_in_chan, values, density = detect_Concordia(dat_orig,
                                                                data.s_freq,
@@ -239,6 +286,7 @@ class DetectSpindle:
             # end of loop over chan
 
         spindle.events = sorted(all_spindles, key=lambda x: x['start'])
+        lg.info(str(len(spindle.events)) + ' spindles detected.')
 
         if self.merge and len(data.axis['chan'][0]) > 1:
             spindle.events = merge_close(spindle.events, self.min_interval)
@@ -249,6 +297,78 @@ class DetectSpindle:
         return spindle
 
 
+def detect_Ray2015(dat_orig, s_freq, time, opts):
+    """Spindle detection based on Ray et al., 2015
+    
+    Parameters
+    ----------
+    dat_orig : ndarray (dtype='float')
+        vector with the data for one channel
+    s_freq : float
+        sampling frequency
+    time : ndarray (dtype='float')
+        vector with the time points for each sample
+    opts : instance of 'DetectSpindle'
+        'det_wavelet' : dict
+            parameters for 'morlet',
+        'smooth' : dict
+            parameters for 'moving_avg'
+        'det_thresh' : float
+            detection threshold
+        'sel_thresh' : nan
+            not used, but keep it for consistency with the other methods
+        'duration' : tuple of float
+            min and max duration of spindles
+
+    Returns
+    -------
+    list of dict
+        list of detected spindles
+    dict
+        'det_value_lo' with detection value, 'det_value_hi' is nan,
+        'sel_value' is nan (for consistency with other methods)
+    float
+        spindle density, per 30-s epoch
+
+    References
+    ----------
+    Ray, L. B. et al. Front. Hum. Neurosci. 9-16 (2015).
+    """
+    dat_det = transform_signal(dat_orig, s_freq, 'butter', opts.det_butter)
+    dat_det = transform_signal(dat_det, s_freq, 'cdemod', opts.cdemod)
+    dat_det = transform_signal(dat_det, s_freq, 'low_butter', 
+                               opts.det_low_butter)
+    dat_det = transform_signal(dat_det, s_freq, 'tri_smooth', opts.smooth)
+    dat_det = transform_signal(dat_det, s_freq, 'abs2')
+    dat_det = transform_signal(dat_det, s_freq, 'zscore', opts.zwin)
+    
+    det_value = opts.det_thresh_lo
+    sel_value = opts.sel_thresh
+    
+    events = detect_events(dat_det, 'above_thresh', det_value)
+    
+    if events is not None:
+        events = select_events(dat_det, events, 'above_thresh', sel_value)
+        
+        events = _merge_close(dat_det, events, time, opts.min_interval)
+        events = within_duration(events, time, opts.duration)
+        events = remove_straddlers(events, time, s_freq)
+
+        power_peaks = peak_in_power(events, dat_orig, s_freq, opts.power_peaks)
+        power_avgs = avg_power(events, dat_orig, s_freq, opts.frequency)
+        sp_in_chan = make_spindles(events, power_peaks, power_avgs, dat_det,
+                                   dat_orig, time, s_freq)
+
+    else:
+        lg.info('No spindle found')
+        sp_in_chan = []
+
+    values = {'det_value_lo': det_value, 'det_value_hi': nan, 'sel_value': nan}
+
+    density = len(sp_in_chan) * s_freq * 30 / len(dat_orig)
+
+    return sp_in_chan, values, density
+
 def detect_Wamsley2012(dat_orig, s_freq, time, opts):
     """Spindle detection based on Wamsley et al. 2012
 
@@ -485,7 +605,7 @@ def detect_Ferrarelli2007(dat_orig, s_freq, time, opts):
     return sp_in_chan, values, density
 
 
-def detect_Moelle2002(dat_orig, s_freq, time, opts):
+def detect_Moelle2011(dat_orig, s_freq, time, opts):
     """Spindle detection based on Moelle et al. 2011
 
     Parameters
@@ -494,9 +614,11 @@ def detect_Moelle2002(dat_orig, s_freq, time, opts):
         vector with the data for one channel
     s_freq : float
         sampling frequency
+    time : ndarray (dtype='float')
+        vector with the time points for each sample
     opts : instance of 'DetectSpindle'
-        'det_luebeck' : dict
-            parameters for 'luebeck',
+        'det_remez' : dict
+            parameters for 'remez',
         'moving_rms' : dict
             parameters for 'moving_rms'
         'smooth' : dict
@@ -518,14 +640,9 @@ def detect_Moelle2002(dat_orig, s_freq, time, opts):
     float
         spindle density, per 30-s epoch
 
-    Notes
-    -----
-    The original article does not specify a filter, but butter seems the best
-    to me.
-
     References
     ----------
-    Moelle, M. et al. Sleep 34, 1411-21 (2011).
+    Moelle, M. et al. J. Neurosci. 22(24), 10941-7 (2002).
     """
     dat_det = transform_signal(dat_orig, s_freq, 'remez', opts.det_remez)
     dat_det = transform_signal(dat_det, s_freq, 'moving_rms', opts.moving_rms)
@@ -557,6 +674,83 @@ def detect_Moelle2002(dat_orig, s_freq, time, opts):
     return sp_in_chan, values, density
 
 
+def detect_FASST(dat_orig, s_freq, time, opts, submethod='rms'):
+    """Spindle detection based on FASST method, itself based on Moelle et al. 
+    (2002).
+    
+    Parameters
+    ----------
+    dat_orig : ndarray (dtype='float')
+        vector with the data for one channel
+    s_freq : float
+        sampling frequency
+    time : ndarray (dtype='float')
+        vector with the time points for each sample
+    opts : instance of 'DetectSpindle'
+        'det_remez' : dict
+            parameters for 'remez',
+        'moving_rms' : dict
+            parameters for 'moving_rms'
+        'smooth' : dict
+            parameters for 'moving_avg'
+        'det_thresh' : float
+            detection threshold
+        'sel_thresh' : nan
+            not used, but keep it for consistency with the other methods
+        'duration' : tuple of float
+            min and max duration of spindles
+    submethod : str
+        'abs' (rectified) or 'rms' (root-mean-square)
+
+    Returns
+    -------
+    list of dict
+        list of detected spindles
+    dict
+        'det_value_lo' with detection value, 'det_value_hi' with nan,
+        'sel_value' with nan
+    float
+        spindle density, per 30-s epoch
+
+    References
+    ----------
+    Leclercq, Y. et al. Compu. Intel. and Neurosci. (2011).
+    """
+    dat_det = transform_signal(dat_orig, s_freq, 'butter', opts.det_butter)
+    
+    det_value = percentile(dat_det, opts.det_thresh_lo)
+    
+    if submethod == 'abs':
+        dat_det = transform_signal(dat_det, s_freq, 'abs')
+    elif submethod == 'rms':
+        dat_det = transform_signal(dat_det, s_freq, 'moving_rms', 
+                                   opts.moving_rms)
+        
+    dat_det = transform_signal(dat_det, s_freq, 'moving_avg', opts.smooth)
+    
+    events = detect_events(dat_det, 'above_thresh', det_value)
+
+    if events is not None:
+        events = within_duration(events, time, opts.duration)
+        events = _merge_close(dat_det, events, time, opts.min_interval)
+        events = remove_straddlers(events, time, s_freq)
+
+        power_peaks = peak_in_power(events, dat_orig, s_freq, opts.power_peaks)
+        power_avgs = avg_power(events, dat_orig, s_freq, opts.frequency)
+        sp_in_chan = make_spindles(events, power_peaks, power_avgs, dat_det,
+                                   dat_orig, time, s_freq)
+
+    else:
+        lg.info('No spindle found')
+        sp_in_chan = []
+
+    values = {'det_value_lo': det_value, 'det_value_hi': nan, 'sel_value': nan}
+
+    density = len(sp_in_chan) * s_freq * 30 / len(dat_orig)
+
+    return sp_in_chan, values, density
+    
+
 def detect_UCSD(dat_orig, s_freq, time, opts):
     """Spindle detection based on the UCSD method
 
@@ -738,6 +932,10 @@ def transform_signal(dat, s_freq, method, method_opt=None):
         order : int
             filter order (will be effecively doubled by filtfilt)
 
+    cdemod has parameters:
+        freq : float
+            carrier frequency for complex demodulation
+    
     cheby2 has parameters:
         freq : tuple of float
             low and high values for bandpass
@@ -793,9 +991,20 @@ def transform_signal(dat, s_freq, method, method_opt=None):
             bandwidth, in hertz, between stop and pass frequencies
         dur : float
             dur * s_freq = N, where N is the filter order, a.k.a number of taps
+                
+    tri_smooth has parameters:
+        dur : float
+            length of convolution window, base of isosceles triangle
+    
+    zscore has parameters:
+        dur : float
+            duration of the z-score sliding window (sec)
     """
     if 'abs' == method:
         dat = absolute(dat)
+        
+    if 'abs2' == method:
+        dat = dat.real**2 + dat.imag**2
 
     if 'butter' == method:
         freq = method_opt['freq']
@@ -806,6 +1015,21 @@ def transform_signal(dat, s_freq, method, method_opt=None):
         b, a = butter(N, Wn, btype='bandpass')
         dat = filtfilt(b, a, dat)
         
+    if 'sosbutter' == method:
+        freq = method_opt['freq']
+        N = method_opt['order']
+
+        nyquist = s_freq / 2
+        Wn = asarray(freq) / nyquist
+        sos = butter(N, Wn, btype='bandpass', output='sos')
+        dat = sosfiltfilt(sos, dat)
+        
+    if 'cdemod' == method:
+        carr_freq = method_opt['freq']
+        carr_sig = exp(-1j * 2 * pi * carr_freq * arange(0, len(dat)) / s_freq)
+        
+        dat = dat * carr_sig        
+    
     if 'cheby2' == method:
         freq = method_opt['freq']
         N = method_opt['order']
@@ -865,11 +1089,12 @@ def transform_signal(dat, s_freq, method, method_opt=None):
     if 'moving_rms' == method:
         dur = method_opt['dur']
         halfdur = int(floor(s_freq * dur / 2))
-        ldat = len(dat)
-        rms = zeros((ldat))
+        lendat = len(dat)
+        rms = zeros((lendat))
 
-        for i in range(ldat):
-            rms[i] = sqrt(mean(square(dat[max(0, i - halfdur):min(ldat, i + halfdur)])))
+        for i in range(lendat):
+            rms[i] = sqrt(mean(square(
+                    dat[max(0, i - halfdur):min(lendat, i + halfdur)])))
         dat = rms
     
     if 'remez' == method:
@@ -898,6 +1123,29 @@ def transform_signal(dat, s_freq, method, method_opt=None):
             x = abs(fftconvolve(dat, one_wm, mode='same'))
             tfr[:, i] = fftconvolve(x, tukey(win), mode='same')
         dat = mean(tfr, axis=1)
+        
+    if 'tri_smooth' == method:
+        T = int(method_opt['dur'] * s_freq / 2)
+        a = arange(T, 0, -1)
+        
+        H = hstack([a[-1:0:-1], a])
+        H = H / sum(H)
+        
+        dat = fftconvolve(dat, H, mode='same')
+    
+    if 'zscore' == method:
+        dur = int(floor(method_opt['dur'] * s_freq))
+        halfdur = int(floor(dur / 2))
+        lendat = len(dat)
+        stop = lendat - dur
+        zs = zeros((lendat))
+        
+        for i in range(lendat):
+            start = min(max(0, i - halfdur), stop)
+            end = min(start + dur, lendat)
+            windat = dat[start:end]
+            zs[i] = (dat[i] - mean(windat)) / std(windat)
+        dat = zs
 
     return dat
 
