@@ -7,10 +7,10 @@ basic elements, use the package "detect" for example.
 
 """
 from .filter import filter_, convolve
-from .select import (select, resample, get_times, _select_channels,
-                     fetch_segments, Segments)
-from .frequency import frequency, timefrequency, get_power
-from .merge import concatenate
+from .select import (select, resample, get_times, _select_channels, fetch, 
+                     Segments)
+from .frequency import frequency, timefrequency, band_power
+from .merge import concatenate, _merge_metadata
 from .math import math, get_descriptives, slopes
 from .montage import montage
 from .peaks import peaks
