@@ -292,9 +292,10 @@ def resample(data, s_freq=None, axis='time', ftype='fir', n=None):
     return output
 
 
-def fetch_segments(dataset, annot, cat=(0, 0, 0, 0), evt_type=None, stage=None, 
-                    cycle=None, chan_full=None, epoch=None, epoch_dur=30, 
-                    min_dur=0, reject_epoch=False, reject_artf=False):
+def fetch(dataset, annot, cat=(0, 0, 0, 0), evt_type=None, stage=None, 
+          cycle=None, chan_full=None, epoch=None, epoch_dur=30, 
+          epoch_overlap=0, epoch_step=None, reject_epoch=False, 
+          reject_artf=False, min_dur=0):
     """Create instance of Segments for analysis, complete with info about 
     stage, cycle, channel, event type. Segments contains only metadata until
     .read_data is called.
@@ -332,8 +333,12 @@ def fetch_segments(dataset, annot, cat=(0, 0, 0, 0), evt_type=None, stage=None,
         If None, longest run of signal is returned.
     epoch_dur : float
         only for epoch='unlocked'. Duration of epochs returned, in seconds.
-    min_dur : float
-        Minimum duration of segments returned, in seconds.
+    epoch_overlap : float
+        only for epoch='unlocked'. Ratio of overlap between two consecutive 
+        segments. Value between 0 and 1. Overriden by step.
+    epoch_step : float
+        only for epoch='unlocked'. Time between consecutive epoch starts, in 
+        seconds. Overrides epoch_overlap/
     reject_epoch: bool
         If True, epochs marked as 'Poor' quality or staged as 'Artefact' will 
         be rejected (and the signal segmented in consequence). Has no effect on
@@ -341,7 +346,8 @@ def fetch_segments(dataset, annot, cat=(0, 0, 0, 0), evt_type=None, stage=None,
     reject_artf : bool
         If True, excludes events marked as 'Artefact' (and signal is segmented
         in consequence).
-        
+    min_dur : float
+        Minimum duration of segments returned, in seconds.        
     """
     bundles = get_times(annot, evt_type=evt_type, stage=stage, cycle=cycle, 
                         chan=chan_full, exclude=reject_epoch)
@@ -362,8 +368,14 @@ def fetch_segments(dataset, annot, cat=(0, 0, 0, 0), evt_type=None, stage=None,
             bundles = _divide_bundles(bundles)
 
         elif 'unlocked' == epoch:
+            
+            if epoch_step is not None:
+                step = epoch_step
+            else:
+                step = epoch_dur - (epoch_dur * epoch_overlap)
+                
             bundles = _concat(bundles, cat)
-            bundles = _find_intervals(bundles, epoch_dur)
+            bundles = _find_intervals(bundles, epoch_dur, step)
                 
         elif not epoch:
             bundles = _concat(bundles, cat)
@@ -578,19 +590,19 @@ def _divide_bundles(bundles):
     return divided
 
 
-def _find_intervals(bundles, interval):
-    """Divide bundles into consecutive segments of a certain duration,
+def _find_intervals(bundles, duration, step):
+    """Divide bundles into segments of a certain duration and a certain step, 
     discarding any remainder."""
     segments = []
     for bund in bundles:
         beg, end = bund['times'][0][0], bund['times'][-1][1]
 
-        if end - beg >= interval:
-            new_begs = arange(beg, end, interval)[:-1]
+        if end - beg >= duration:
+            new_begs = arange(beg, end, step)
 
             for t in new_begs:
                 seg = bund.copy()
-                seg['times'] = [(t, t + interval)]
+                seg['times'] = [(t, t + duration)]
                 segments.append(seg)
 
     return segments
